# Step to make a call to chromadb collection to fetch relevant chunks

from typing import Union, Dict, Any, List, Optional, Literal
from athina.steps import Step
import chromadb
from chromadb.config import Settings
from enum import Enum
from jinja2 import Environment
from athina.helpers.jinja_helper import PreserveUndefined
from chromadb.utils.embedding_functions import OpenAIEmbeddingFunction
import time


class AuthType(str, Enum):
    TOKEN = "token"
    BASIC = "basic"
    NONE = "none"


class ChromaRetrieval(Step):
    """
    Step that retrieves documents from an existing Chroma collection.

    Attributes:
        host (str): The host of the Chroma server.
        port (int): The port of the Chroma server.
        collection_name (str): The name of the Chroma collection.
        limit (int): The maximum number of results to fetch.
        user_query (str): the query which will be sent to chroma.
        openai_api_key (str): The OpenAI API key.
        auth_type (str): The authentication type for the Chroma server (e.g., "token" or "basic").
        auth_credentials (str): The authentication credentials for the Chroma server.
    """

    host: str
    port: int
    collection_name: str
    limit: int
    user_query: str
    openai_api_key: str
    auth_type: Optional[AuthType] = None
    auth_credentials: Optional[str] = None
    env: Environment = None
    _client: chromadb.Client = None
    _collection: chromadb.Collection = None
    _embedding_function = None

    class Config:
        arbitrary_types_allowed = True

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        settings = None

        if self.auth_type == "none":
            settings = None
        elif self.auth_type is not None:
            auth_provider = {
                AuthType.TOKEN: "chromadb.auth.token_authn.TokenAuthClientProvider",
                AuthType.BASIC: "chromadb.auth.basic_authn.BasicAuthClientProvider",
            }.get(self.auth_type)

            if auth_provider and self.auth_credentials:
                settings = Settings(
                    chroma_client_auth_provider=auth_provider,
                    chroma_client_auth_credentials=self.auth_credentials,
                )
        else:
            settings = None

        self._client = chromadb.HttpClient(
            host=self.host, port=self.port, settings=settings
        )
        self._embedding_function = OpenAIEmbeddingFunction(api_key=self.openai_api_key)
        self._collection = self._client.get_collection(
            name=self.collection_name, embedding_function=self._embedding_function
        )

    """Makes a call to chromadb collection to fetch relevant chunks"""

    def execute(self, input_data: Any) -> Union[Dict[str, Any], None]:
        start_time = time.perf_counter()

        if input_data is None or not isinstance(input_data, dict):
            return self._create_step_result(
                status="error",
                data="Input data must be a dictionary.",
                start_time=start_time,
            )

        self.env = self._create_jinja_env()

        query_text = self.env.from_string(self.user_query).render(**input_data)

        if query_text is None:
            return self._create_step_result(
                status="error", data="Query text is Empty.", start_time=start_time
            )

        try:
            response = self._collection.query(
                query_texts=[query_text],
                n_results=self.limit,
                include=["documents", "metadatas", "distances"],
            )
            result = [
                {"text": text, "score": distance}
                for text, distance in zip(
                    response["documents"][0], response["distances"][0]
                )
            ]
            return self._create_step_result(
                status="success",
                data=result,
                start_time=start_time,
            )
        except Exception as e:
            return self._create_step_result(
                status="error",
                data=str(e),
                start_time=start_time,
            )

    def close(self):
        if self._client:
            self._client = None



from typing import List, Optional
from ..llm_evaluator import LlmEvaluator
from .examples import DOES_RESPONSE_ANSWER_QUERY_EVAL_EXAMPLES
from athina.evals.eval_type import LlmEvalTypeId
from athina.metrics.metric_type import MetricType


class DoesResponseAnswerQuery(LlmEvaluator):
    """
    This evaluator checks if the response answers specifically what the user is asking about, and covers all aspects of the user's query.
    """

    SYSTEM_MESSAGE_TEMPLATE = """
    You are an expert at evaluating whether the response answers specifically what the user is asking about, and covers all aspects of the user's query.
    You are not checking for correctness, or factual accuracy. You are only checking if the response answers the user's query.
    """

    USER_MESSAGE_TEMPLATE = """
        Let's think step by step.
        1. Consider the following: 
        user's query: {query}.
        response: {response}.
        2. Determine if the response answers specifically what the user is asking about, and covers all aspects of the user's query.
        3. Provide a brief explanation of why the response does or does not answer the user's query sufficiently, labeled as 'explanation', leading up to a verdict (Pass/Fail) labeled as 'result'.
        4. Return a JSON object in the following format: "result": 'result', "explanation": 'explanation'

        ### EXAMPLES ###
        Here's are some examples: 
        {examples}
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    @property
    def name(self):
        return LlmEvalTypeId.DOES_RESPONSE_ANSWER_QUERY.value

    @property
    def display_name(self):
        return "Does Response Answer Query"

    @property
    def default_model(self):
        return "gpt-4-1106-preview"

    @property
    def required_args(self):
        return ["query", "response"]

    @property
    def examples(self):
        return DOES_RESPONSE_ANSWER_QUERY_EVAL_EXAMPLES

    @property
    def metric_ids(self) -> List[str]:
        return [MetricType.PASSED.value]

    def is_failure(self, result) -> Optional[bool]:
        return bool(str(result).lower() == "fail")

    def _user_message(
        self,
        query: str,
        response: str,
        **kwargs,
    ) -> str:
        """
        Generates data for evaluation.

        :param query: user query
        :param response: llm response
        :return: A dictionary with formatted data for evaluation
        """
        return self.USER_MESSAGE_TEMPLATE.format(
            query=query,
            response=response,
            examples=self._examples_str(),
        )



from typing import Union, Dict, Any
from athina.steps import Step
import json
from jsonpath_ng import parse
import time


class ExtractJsonPath(Step):
    """
    Step that extracts json path from text using the JsonPath provided to the step.

    Attributes:
        input_column: The row's column to extract JsonPath from.
        json_path: The JsonPath to extract from the text.
    """

    input_column: str
    json_path: str

    def execute(self, input_data: Any) -> Union[Dict[str, Any], None]:
        """Extract the JsonPath from the input data."""
        start_time = time.perf_counter()

        if input_data is None:
            input_data = {}

        if not isinstance(input_data, dict):
            return self._create_step_result(
                status="error",
                data="Input data must be a dictionary.",
                start_time=start_time,
            )
        input_text = input_data.get(self.input_column, None)

        if input_text is None:
            return self._create_step_result(
                status="error",
                data="Input column not found.",
                start_time=start_time,
            )

        try:
            if isinstance(input_text, dict) or isinstance(input_text, list):
                input_json = input_text
            elif isinstance(input_text, str):
                input_json = json.loads(input_text)
            else:
                return self._create_step_result(
                    status="error",
                    data="Input column must be a dictionary or a string.",
                    start_time=start_time,
                )
            result = parse(self.json_path).find(input_json)

            if not result or len(result) == 0:
                result = None
            elif len(result) == 1:
                result = result[0].value
            else:
                result = [match.value for match in result]

            return self._create_step_result(
                status="success",
                data=result,
                start_time=start_time,
            )
        except Exception as e:
            return self._create_step_result(
                status="error",
                data=str(e),
                start_time=start_time,
            )



import inspect


def get_named_parameters(func):
    """
    Get all named parameters of a function.
    """
    parameters = inspect.signature(func).parameters
    named_parameters = [
        param
        for param in parameters
        if parameters[param].default != inspect.Parameter.empty
    ]
    return named_parameters


def get_named_non_default_parameters(func):
    """
    Get all named parameters without default values of a function.
    """
    parameters = inspect.signature(func).parameters
    named_non_default_parameters = [
        param
        for param in parameters
        if parameters[param].default == inspect.Parameter.empty
    ]
    return named_non_default_parameters



import requests
import time
from typing import List, Optional
from ....keys import OpenAiApiKey
from ...base_evaluator import BaseEvaluator
from ....metrics.metric_type import MetricType
from ....evals.eval_type import FunctionEvalTypeId
from ....errors.exceptions import NoOpenAiApiKeyException
from ....interfaces.result import EvalResult, EvalResultMetric


class OpenAiContentModeration(BaseEvaluator):
    @property
    def name(self):
        return FunctionEvalTypeId.OPENAI_CONTENT_MODERATION.value

    @property
    def display_name(self):
        return "OpenAI Content Moderation"

    @property
    def metric_ids(self) -> List[str]:
        return [MetricType.PASSED.value]

    @property
    def required_args(self):
        return ["text"]

    @property
    def examples(self):
        return None

    def __init__(self, open_ai_api_key: Optional[str] = None):
        if open_ai_api_key is None:
            if OpenAiApiKey.get_key() is None:
                raise NoOpenAiApiKeyException()
            self.open_ai_api_key = OpenAiApiKey.get_key()
        else:
            self.open_ai_api_key = open_ai_api_key

    def is_failure(self, content_moderation_response: dict) -> Optional[bool]:
        results = content_moderation_response.get("results", [])
        if results and len(results) > 0:
            # If the result is flagged, return True else False
            return bool(results[0].get("flagged", False))
        # Assuming when no results are returned, it is not a failure
        return False

    def get_reason(self, content_moderation_response: dict) -> Optional[str]:
        results = content_moderation_response.get("results", [])
        if results and len(results) > 0 and results[0].get("flagged", False):
            result = results[0]
            if results[0].get("flagged", False):
                flagged_categories = [
                    category
                    for category, flagged in result["categories"].items()
                    if flagged
                ]
                # Form a comma-separated string of flagged categories
                reason = ", ".join(flagged_categories)
                return f"The text was flagged in these categories: {reason}"
        return "The text was not flagged"

    def _evaluate(self, **kwargs) -> EvalResult:
        # Start timer
        start_time = time.perf_counter()
        self.validate_args(**kwargs)
        text = kwargs["text"]
        content_moderation_response = self.get_content_moderation_result(text)
        failure = self.is_failure(content_moderation_response)
        reason = self.get_reason(content_moderation_response)
        end_time = time.perf_counter()
        # Calculate runtime
        runtime = (end_time - start_time) * 1000

        return EvalResult(
            name=self.name,
            display_name=self.display_name,
            data={"text": text},
            failure=failure,
            reason=reason,
            runtime=int(runtime),
            model=None,
            metrics=[
                EvalResultMetric(id=MetricType.PASSED.value, value=float(not failure))
            ],
        )

    # EXAMPLE RESPONSE JSON
    # {
    #     "id": "modr-XXXXX",
    #     "model": "text-moderation-007",
    #     "results": [
    #         {
    #             "flagged": true,
    #             "categories": {
    #                 "sexual": false,
    #                 "hate": false,
    #                 "harassment": false,
    #                 "self-harm": false,
    #                 "sexual/minors": false,
    #                 "hate/threatening": false,
    #                 "violence/graphic": false,
    #                 "self-harm/intent": false,
    #                 "self-harm/instructions": false,
    #                 "harassment/threatening": true,
    #                 "violence": true
    #             },
    #             "category_scores": {
    #                 "sexual": 1.2282071e-6,
    #                 "hate": 0.010696256,
    #                 "harassment": 0.29842457,
    #                 "self-harm": 1.5236925e-8,
    #                 "sexual/minors": 5.7246268e-8,
    #                 "hate/threatening": 0.0060676364,
    #                 "violence/graphic": 4.435014e-6,
    #                 "self-harm/intent": 8.098441e-10,
    #                 "self-harm/instructions": 2.8498655e-11,
    #                 "harassment/threatening": 0.63055265,
    #                 "violence": 0.99011886
    #             }
    #         }
    #     ]
    # }

    def get_content_moderation_result(self, text: str):
        # Define the endpoint URL
        url = "https://api.openai.com/v1/moderations"
        # Prepare headers and data payload for the HTTP request
        headers = {
            "Authorization": f"Bearer {self.open_ai_api_key}",
            "Content-Type": "application/json",
        }
        data = {"input": text}
        # Make the HTTP POST request
        response = requests.post(url, json=data, headers=headers)
        if response.status_code == 200:
            return response.json()
        else:
            raise Exception(
                f"Error occurred during OpenAI Content Moderation: {response}"
            )



from ..example import FewShotExample, FewShotExampleInputParam

CONTEXT_CONTAINS_ENOUGH_INFORMATION_EXAMPLES = [
    FewShotExample(
        input_params=[
            FewShotExampleInputParam(
                name="context",
                value="Bjarne Stroustrup invented C++",
            ),
            FewShotExampleInputParam(
                name="query",
                value="Who invented the linux os?",
            ),
        ],
        eval_result="Fail",
        eval_reason="The context does not provide any relevant information about the Linux OS or its inventor.",
    ),
    FewShotExample(
        input_params=[
            FewShotExampleInputParam(
                name="context",
                value="In 1969, Neil Armstrong became the first person to walk on the moon.",
            ),
            FewShotExampleInputParam(
                name="query",
                value="What was the name of the spaceship used for the moon landing in 1969?",
            ),
        ],
        eval_result="Fail",
        eval_reason="The context provided does not include any information about the name of the spaceship used for the moon landing. The query specifically asks for the name of the spaceship, which is not present in the context.",
    ),
    FewShotExample(
        input_params=[
            FewShotExampleInputParam(
                name="context",
                value="YC is a seed stage accelerator program. It was founded in 2005 by Paul Graham, Jessica Livingston, Trevor Blackwell, and Robert Tappan Morris.",
            ),
            FewShotExampleInputParam(
                name="query",
                value="How much does YC invest in startups?",
            ),
        ],
        eval_result="Fail",
        eval_reason="The context does not include any information about the amount YC invests in startups.",
    ),
]



from typing import List, Optional, Dict
from athina.interfaces.athina import AthinaFilters


class ConversationLoader:
    """
    This class is a data loader for conversation data

    Attributes:
        raw_dataset: The raw dataset as loaded from the source.
        processed_dataset: The processed dataset is the list of strings
    """

    def __init__(
        self,
    ):
        """
        Initializes the loader with specified or default column names.
        """
        self._raw_dataset = {}
        self._processed_dataset = []

    def load_athina_inferences(
        self,
        filters: Optional[AthinaFilters] = None,
        limit: int = 10,
        context_key: Optional[str] = None,
    ):
        """
        Load data from Athina API.
        """
        pass

    def load_from_string_array(self, strings: List[str]):
        """
        Loads data from a list of strings.

        :param strings: List of strings to be loaded.
        """
        if strings is None or not all(isinstance(s, str) for s in strings):
            raise ValueError("Input must be a list of strings")

        self._processed_dataset.extend(strings)

    def load_from_openai_messages(self, messages: List[List[Dict[str, str]]]):
        """
        Processes and loads data from an array of lists containing messages.

        :param messages: Array of lists of messages with roles and content.
        """
        if not all(isinstance(msg_list, list) for msg_list in messages):
            raise ValueError("Input must be an array of lists")

        for msg_list in messages:
            for msg in msg_list:
                if (
                    not isinstance(msg, dict)
                    or "role" not in msg
                    or "content" not in msg
                ):
                    raise ValueError(
                        "Each message must be a dict with 'role' and 'content' keys"
                    )
                prefix = "AI: " if msg["role"] == "assistant" else "User: "
                self._processed_dataset.append(prefix + msg["content"])



from typing import Any, Dict, Optional
import requests
from athina.steps import Step
import time


class TranscribeSpeechToText(Step):
    """
    Step that transcribes audio to text using specified model.

    Attributes:
        audio_url: URL of the audio file to transcribe
        language: Language of the audio (optional)
        model: Model to use for transcription
        api_key: Deepgram API key
        profanity_filter: Remove profanity from transcript
        punctuate: Add punctuation and capitalization
        redact: Redact sensitive information
        replace: Terms to replace
        search: Terms to search for
        detect_language: Detect audio language
        filler_words: Include filler words
        diarize: Enable speaker diarization
        dictation: Convert spoken punctuation commands
    """

    audio_url: str
    language: Optional[str] = "en"
    model: str
    api_key: str
    profanity_filter: bool = False
    punctuate: bool = False
    redact: Optional[str] = None
    replace: Optional[str] = None
    search: Optional[str] = None
    detect_language: bool = False
    filler_words: bool = False
    diarize: bool = False
    dictation: bool = False

    def execute(self, input_data: Any) -> Dict[str, Any]:
        """Transcribe audio file and return the text."""

        start_time = time.perf_counter()
        try:
            # Prepare the request to Deepgram API
            headers = {
                "Authorization": f"Token {self.api_key}",
                "Content-Type": "application/json",
            }

            payload = {"url": self.audio_url}

            # Build URL parameters
            params = {}
            if self.model is not None:
                params["model"] = self.model
            if self.language is not None:
                params["language"] = self.language

            # Add new parameters
            if self.profanity_filter:
                params["profanity_filter"] = "true"
            if self.punctuate:
                params["punctuate"] = "true"
            if self.redact:
                for item in self.redact.split(","):
                    params["redact"] = item.strip()
            if self.replace:
                for replacement in self.replace.split(","):
                    params["replace"] = replacement.strip()
            if self.search:
                for term in self.search.split(","):
                    params["search"] = term.strip()
            if self.detect_language:
                params["detect_language"] = "true"
            if self.filler_words:
                params["filler_words"] = "true"
            if self.diarize:
                params["diarize"] = "true"
            if self.dictation:
                params["dictation"] = "true"

            # Make request to Deepgram API
            response = requests.post(
                "https://api.deepgram.com/v1/listen",
                headers=headers,
                json=payload,
                params=params,
                timeout=30,
            )
            response.raise_for_status()

            # Parse the response
            result = response.json()

            # Extract the transcript
            transcribed_text = (
                result.get("results", {})
                .get("channels", [{}])[0]
                .get("alternatives", [{}])[0]
                .get("transcript", "")
            )

            # Create a simplified metadata object that's JSON serializable
            metadata = {
                "duration": result.get("metadata", {}).get("duration"),
                "channels": result.get("metadata", {}).get("channels"),
                "model": result.get("metadata", {}).get("model"),
                "language": result.get("metadata", {}).get("language"),
            }

            return self._create_step_result(
                status="success",
                data=transcribed_text,
                metadata=metadata,
                start_time=start_time,
            )

        except requests.RequestException as e:
            return self._create_step_result(
                status="error",
                data=f"Failed to download audio file: {str(e)}",
                start_time=start_time,
            )

        except Exception as e:
            return self._create_step_result(
                status="error",
                data=f"Transcription failed: {str(e)}",
                start_time=start_time,
            )



# Step to make an external api call
import json
import time
from typing import Union, Dict, Any, Optional
import aiohttp
from athina.steps.base import Step
import asyncio
from jinja2 import Environment


def prepare_template_data(
    env: Environment,
    template_dict: Optional[Dict[str, str]],
    input_data: Dict[str, Any],
) -> Optional[Dict[str, str]]:
    """Prepare template data by rendering Jinja2 templates."""
    if template_dict is None:
        return None

    prepared_dict = template_dict.copy()
    for key, value in prepared_dict.items():
        prepared_dict[key] = env.from_string(value).render(**input_data)
    return prepared_dict


def debug_json_structure(body_str: str, error: json.JSONDecodeError) -> dict:
    """Analyze JSON structure and identify problematic keys."""
    lines = body_str.split("\n")
    error_line_num = error.lineno - 1

    return {
        "original_body": body_str,
        "problematic_line": (
            lines[error_line_num] if error_line_num < len(lines) else None
        ),
    }


def prepare_body(
    env: Environment, body_template: Optional[str], input_data: Dict[str, Any]
) -> Optional[str]:
    """Prepare request body by rendering Jinja2 template."""
    if body_template is None:
        return None

    return env.from_string(body_template).render(**input_data)


class ApiCall(Step):
    """
    Step that makes an external API call.

    Attributes:
        url: The URL of the API endpoint to call.
        method: The HTTP method to use (e.g., 'GET', 'POST', 'PUT', 'DELETE').
        headers: Optional headers to include in the API request.
        params: Optional params to include in the API request.
        body: Optional request body to include in the API request.
    """

    url: str
    method: str
    headers: Optional[Dict[str, str]] = None
    params: Optional[Dict[str, str]] = None
    body: Optional[str] = None
    env: Environment = None
    name: Optional[str] = None
    timeout: int = 30  # Default timeout in seconds
    retries: int = 2  # Default number of retries

    class Config:
        arbitrary_types_allowed = True

    def process_response(
        self,
        status_code: int,
        response_text: str,
        start_time: float,
    ) -> Dict[str, Any]:
        """Process the API response and return formatted result."""
        if status_code >= 400:
            # If the status code is an error, return the error message
            return self._create_step_result(
                status="error",
                data=f"Failed to make the API call.\nStatus code: {status_code}\nError:\n{response_text}",
                start_time=start_time,
            )

        try:
            json_response = json.loads(response_text)
            # If the response is JSON, return the JSON data
            return self._create_step_result(
                status="success",
                data=json_response,
                start_time=start_time,
            )
        except json.JSONDecodeError:
            # If the response is not JSON, return the text
            return self._create_step_result(
                status="success",
                data=response_text,
                start_time=start_time,
            )

    async def execute_async(self, input_data: Any) -> Union[Dict[str, Any], None]:
        """Make an async API call and return the response."""
        start_time = time.perf_counter()

        if input_data is None:
            input_data = {}

        if not isinstance(input_data, dict):
            return self._create_step_result(
                status="error",
                data="Input data must be a dictionary.",
                start_time=start_time,
            )
        # Prepare the environment and input data
        self.env = self._create_jinja_env()

        # Prepare request components
        prepared_body = prepare_body(self.env, self.body, input_data)
        prepared_headers = prepare_template_data(self.env, self.headers, input_data)
        prepared_params = prepare_template_data(self.env, self.params, input_data)
        # Prepare the URL by rendering the template
        prepared_url = self.env.from_string(self.url).render(**input_data)

        timeout = aiohttp.ClientTimeout(total=self.timeout)

        for attempt in range(self.retries):
            try:
                async with aiohttp.ClientSession(timeout=timeout) as session:
                    try:
                        json_body = (
                            json.loads(prepared_body, strict=False)
                            if prepared_body
                            else None
                        )
                    except json.JSONDecodeError as e:
                        debug_info = debug_json_structure(prepared_body, e)
                        return self._create_step_result(
                            status="error",
                            data=json.dumps(
                                {
                                    "message": f"Failed to parse request body as JSON",
                                    "error_type": "JSONDecodeError",
                                    "error_details": str(e),
                                    "debug_info": debug_info,
                                },
                                indent=2,
                            ),
                            start_time=start_time,
                        )

                    async with session.request(
                        method=self.method,
                        url=prepared_url,
                        headers=prepared_headers,
                        params=prepared_params,
                        json=json_body,
                    ) as response:
                        response_text = await response.text()
                        return self.process_response(
                            response.status, response_text, start_time
                        )

            except asyncio.TimeoutError:
                if attempt < self.retries - 1:
                    await asyncio.sleep(2)
                    continue
                # If the request times out after multiple attempts, return an error message
                return self._create_step_result(
                    status="error",
                    data="Failed to make the API call.\nRequest timed out after multiple attempts.",
                    start_time=start_time,
                )
            except Exception as e:
                # If an exception occurs, return the error message
                return self._create_step_result(
                    status="error",
                    data=f"Failed to make the API call.\nError: {e.__class__.__name__}\nDetails:\n{str(e)}",
                    start_time=start_time,
                )

    def execute(self, input_data: Any) -> Union[Dict[str, Any], None]:
        """Synchronous execute api call that runs the async method in an event loop."""
        return asyncio.run(self.execute_async(input_data))



# Guardrails Restrict To Topic
# https://hub.guardrailsai.com/validator/tryolabs/restricttotopic

import os
import time
from typing import List, Optional, Dict
from athina.interfaces.result import EvalResult, EvalResultMetric
from athina.helpers.logger import logger
from athina.errors.exceptions import NoOpenAiApiKeyException
from athina.keys import OpenAiApiKey
from ...base_evaluator import BaseEvaluator
from athina.metrics.metric_type import MetricType


# Passes when the text is restricted to the specified topics, fails when the text doesn't.
class RestrictToTopic(BaseEvaluator):
    _valid_topics: List[str]
    _invalid_topics = []

    def __init__(
        self,
        valid_topics: List[str],
        invalid_topics: List[str] = [],
        open_ai_api_key: Optional[str] = None,
    ):
        from guardrails.hub import RestrictToTopic

        if open_ai_api_key is None:
            if OpenAiApiKey.get_key() is None:
                raise NoOpenAiApiKeyException()
            os.environ["OPENAI_API_KEY"] = OpenAiApiKey.get_key()
        else:
            self.open_ai_api_key = open_ai_api_key
        self._valid_topics = valid_topics
        self._invalid_topics = invalid_topics

        # Initialize Validator
        self.validator = RestrictToTopic(
            valid_topics=self._valid_topics,
            invalid_topics=self._invalid_topics,
            disable_classifier=True,
            disable_llm=False,
            on_fail="noop",
        )

    @property
    def name(self) -> str:
        return "RestrictToTopic"

    @property
    def display_name(self) -> str:
        return "Restrict To Topic"

    @property
    def metric_ids(self) -> List[str]:
        return [MetricType.PASSED.value]

    @property
    def required_args(self) -> List[str]:
        return ["response"]

    @property
    def examples(self):
        pass

    def to_config(self) -> Optional[Dict]:
        return None

    def is_failure(self, result: bool) -> bool:
        return not (bool(result))

    def _evaluate(self, **kwargs) -> EvalResult:
        from guardrails import Guard

        """
        Run the Guardrails evaluator.
        """
        start_time = time.time()
        self.validate_args(**kwargs)
        metrics = []
        try:
            text = kwargs["response"]
            # Setup Guard
            guard = Guard.from_string(validators=[self.validator])
            validation_passed = False
            # Pass LLM output through guard
            try:
                guard_result = guard.parse(text)
                validation_passed = guard_result.validation_passed
                grade_reason = (
                    "Text is restricted to the specified topics"
                    if validation_passed
                    else "Text is not restricted to the specified topics"
                )
            except Exception as e:
                validation_passed = False
                grade_reason = str(e).replace(
                    "Validation failed for field with errors:", ""
                )

            # Boolean evaluator
            metrics.append(
                EvalResultMetric(
                    id=MetricType.PASSED.value,
                    value=float(validation_passed),
                )
            )
        except Exception as e:
            logger.error(f"Error occurred during eval: {e}")
            raise e

        end_time = time.time()
        eval_runtime_ms = int((end_time - start_time) * 1000)
        llm_eval_result = EvalResult(
            name=self.name,
            display_name=self.display_name,
            data=kwargs,
            failure=self.is_failure(validation_passed),
            reason=grade_reason,
            runtime=eval_runtime_ms,
            model="gpt-3.5-turbo",
            metrics=metrics,
        )
        return {k: v for k, v in llm_eval_result.items() if v is not None}
